/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| (C) Copyright IBM Corp. 2005.  All Rights Reserved.                    |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

package com.ibm.cic.common.core.model;

import java.util.Set;

// An IFeatureGroup represents a hierarchy and a grouping of installable content
// in the context of an offering.
//
// The kind of the feature group determines whether the group is optional
// or required and whether the feature is or is not user visible.
//
// A feature group has a name and description that may be user visible.
//
// A feature group has a collection of nested feature groups capturing the tree
// of features of an offering.
//
// A feature group has a collection of features that identify the installable
// content represented by the group.
//
public interface IFeatureGroup {

    public IIdentity getIdentity();
    
	public FeatureKind getKind();	
	public void getKind(FeatureKind kind);

    public Information getInformation();
    public void setInformation(Information info);
    
    public Set getGroups();
    public void addGroup (IFeatureGroup group);
    
    public Set getFeatures();
    public void addFeature (IFeature feature);

}
